##-- R code for beach lab --##

# This assigment implements descriptive and inferential methods
# for point patterns; including those for kernel smoothing, nearest 
# neighbor methods, and interevent distance methods.  
#
# Before starting the lab make sure that you have used the "change directory"
# command to point toward the subdirectory containing the file "beach.txt".  
# To do this, use the pull-down menus in R, File > Change Directory, and then
# Browse to the correct subdirectory.
#
# This lab was originally written by Stuart Sweeney for an introductory spatial analysis
# course at UC Santa Barbara, Department of Geography.  Other assignments using the same
# beach data can be found at: http://www.geog.ucsb.edu/~sweeney/g172/g172.html.

############### Part I. Point Pattern Description ############
# Use the following command to read in the spatstat library.
################################################################

library(spatstat)

# Read in the beach data and look at summary statistics.

beach<-read.table("beach.txt")
names(beach) 
beach$Xn<-(beach$X-6)*.05
beach$Yn<-(beach$Y+3561)*.05
summary(beach)


# The following command creates an unmarked point pattern data set.
# spatstat uses the data type ppp as input to most functions.
# Skim the file help(ppp)

b.pp<-ppp(beach$Xn,beach$Yn,c(0,150),c(0,180))
summary(b.pp)  # Gives a summary of the PPP data
plot(b.pp)     # Plots the PPP data
identify(b.pp) # Allows the user to query the points in the PPP
# right click on the picture and select "stop" to end the identify

# The first thing we would like to do is check the condition that lambda=lambda(s).
# That is, to assess whether the point pattern has a spatially varying intensity.
# As noted in class we can use kernel smoothing.  Skim the file help(density.ppp).
# Note that kernel smoothing arguements are density(data,std.dev). Execute the following.

plot(density.ppp(b.pp,5))

# Recall from last week how to create multipanel plots.
# Construct an four panel plot using the std. dev. =2,5,10,25
# Interpret the four plots.  What do they show?  Which is most 
# useful and why? Write the code and your answer to the other questions in the space
# below.

## Answer:




# We discussed the nearest neighbor stat G(s) and interevent statistic K(s)
# in class.  The following commands generate objects containing output statistics
# for those two functions. 

G<-Gest(b.pp)
K<-Kest(b.pp)
G # look at the contents of G
K # look at the contents of K
par(mfrow=c(2,1))
plot(G,cbind(theo,rs,raw)~r)
plot(K,sqrt(cbind(theo,border)/pi)-r~r)
par(mfrow=c(1,1))

## Interpret G and K from the plots just generated:



# Recall that the kernel smoothed plot showed pronounced firt-order
# variation.  Spatstat allows several methods for incorporating
# first-order variation into analysis. One of the simplest methods
# is the function kinhom().  Execute the following commands.

smo10<-density.ppp(b.pp,10)
lambda <- smo10[b.pp]
Ki10<-Kinhom(b.pp,lambda)

# or using a stdev=25 and only one line.

Ki25<-Kinhom(b.pp,lambda=density.ppp(b.pp,25)[b.pp])

par(mfrow=c(2,2))
plot(smo10)
plot(density.ppp(b.pp,25))
plot(Ki10,sqrt(cbind(theo,border)/pi)-r~r)
plot(Ki25,sqrt(cbind(theo,border)/pi)-r~r)
par(mfrow=c(1,1))

## Interpret the results contained in the four panel plot you just
## generated:




###################### Part II. Inference ##############################
# So far we have only developed the descriptive measures.
# The function envelope() allows us to assess whether apparent clustering
# or dispersion is signifcant relative to CSR or other null hypotheses.
# Look quickly at the help files for envelope(). Execute the functions below.
#######################################################################

G<-envelope(b.pp,Gest)
K<-envelope(b.pp,Kest)
par(mfrow=c(2,1))
plot(G,cbind(theo,obs,lo,hi)~r)
plot(K,sqrt(cbind(theo,obs,lo,hi)/pi)-r~r)
par(mfrow=c(2,1))

## Interpret the results in the two panel plot:



# Creating envelopes for the inhomogeneous K is a bit more work.  You need to
# create your own function and then call it in the envelope function.  It looks
# as follows. 
smo <- density.ppp(b.pp,25)
myKfun <- function(S, ..., lam) { Kinhom(S, lambda=lam[S], ...) }
Ken <- envelope(b.pp, myKfun, nsim=199,
                simulate=expression(rpoispp(smo)),
                lam=smo, correction="border")
par(mfrow=c(2,1))
plot(Ken,sqrt(cbind(mmean,obs,lo,hi)/pi)-r~r)
plot(Ken[Ken$r<10],sqrt(cbind(mmean,obs,lo,hi)/pi)-r~r) # this shows only r<10.
par(mfrow=c(1,1))

## Interpret the result from the two panel plot:



# Now suppose we would like to look at a subset of the data where people are not
# in the water.  We will assume that the water occupies Y={0,67} and sand, Y={67,180}
# Create a subset of the pp data by running the following.

sand<-beach[beach$Yn>67,]
s.pp<-ppp(sand$Xn,sand$Yn,c(0,150),c(66,180))
plot(s.pp)

# Create a 4-panel plot showing kernel smoothed images with std=7,10 and
# a plot of K and G both with envelopes.  Interpret those plots relative to
# the results for the whole point pattern.

## Answer:




################## Part III. Marked point patterns ####################
# It is common to encounter point patterns with two or more types of 
# discrete entities.  For this section we will work only with the beach
# segment of the pattern.  Recall that our beach data includes several
# binary indicators -- Sit, Sunny, ... You can use names(beach) to see
# all the variable names.
#######################################################################

# The basic command for creating a marked point pattern is to include it
# in the ppp() command as follows.

win.s<-owin(c(0,150),c(66,180))
spp.suit<-ppp(sand$Xn,sand$Yn,window=win.s,marks=factor(sand$Suit))
summary(spp.suit)
plot(spp.suit)


# To produce plots of each marked component separately use:

plot(split(spp.suit))

# Inference on the marked process entails more complicated code but recall
# from the book that the interpretation is very natural.  The following code
# produces cross-K and cross-G with envelopes.

par(mfrow=c(2,2))
plot(spp.suit)
plot(rmpoispp(c(0.00228,0.00801),win=win.s,types=c(0,1)))

FKcr<- function(S, ...) { Kcross(S,"0","1", ...) }
Kc <-envelope(spp.suit, FKcr, nsim=199, simulate=
     expression(rmpoispp(c(0.00228,0.00801),win=win.s,types=c(0,1))))
plot(Kc,sqrt(cbind(mmean,obs,lo,hi)/pi)-r~r) 
 
FGcr<- function(S, ...) { Gcross(S,"0","1", ...) }
Gc <-envelope(spp.suit, FGcr, nsim=199, simulate=
     expression(rmpoispp(c(0.00228,0.00801),win=win.s,types=c(0,1))))
plot(Gc,(cbind(mmean,obs,lo,hi)~r))  
par(mfrow=c(1,1))

## Interpret the results from the plots:
 





# Now, carry out your own analysis using a different set of marks on the data.
# That is, pick one of other variables from names(beach) and rerun the analysis 
# from above.

## Answer:



############## END of Assignment 4 ##########################

# For fun I've included a script below that carries out
# quadrat analysis inference using MC simulations.
# You can look at it to the extant you are interested.

####################################################
##### Quadrat count analysis using MC signif. ######
####################################################

suit<-sand[sand$Orient==0,]
suit.pp<-ppp(suit$Xn,suit$Yn,window=win.s)
summary(suit.pp)

### begin parms ###
###################
nsim<-199
pp<- suit.pp
win.pp<- as.owin(summary(pp)$window)
lambda<-summary(pp)$intensity
rquad<-10
cquad<-10
####################
#### end parms #####

t1<-quadratcount(pp,rquad,cquad)
o<-as.vector(table(c(as.vector(t1),0:(max(t1)+5)))-1)
test<-numeric(0)
tot<-numeric((max(t1)+5))
for (i in 1:nsim) {
   t<-quadratcount(rpoispp(lambda,win=win.pp),rquad,cquad)
   e<-as.vector(table(c(as.vector(t),0:(max(t1)+5)))-1)
   tot<-tot+e }
e<-round(tot/nsim)

for (i in 1:nsim) {
   t<-quadratcount(rpoispp(lambda,win=win.pp),rquad,cquad)
   e1<-as.vector(table(c(as.vector(t),0:(max(t1)+5)))-1)
   test<-c(test,sum((e1[e>0]-e[e>0])**2/e[e>0])) }

o.test<-sum((o[e>0]-e[e>0])**2/e[e>0])
rank<-rank(c(test,o.test))[nsim+1]
alpha<-rank/(1+nsim)
print(c(o.test,alpha,1-alpha))
   

       